@@ -1,23 +1,26 @@
 /*
  * Copyright © 2012 Intel Corporation
  *
- * Permission to use, copy, modify, distribute, and sell this software and its
- * documentation for any purpose is hereby granted without fee, provided that
- * the above copyright notice appear in all copies and that both that copyright
- * notice and this permission notice appear in supporting documentation, and
- * that the name of the copyright holders not be used in advertising or
- * publicity pertaining to distribution of the software without specific,
- * written prior permission.  The copyright holders make no representations
- * about the suitability of this software for any purpose.  It is provided "as
- * is" without express or implied warranty.
+ * Permission is hereby granted, free of charge, to any person obtaining
+ * a copy of this software and associated documentation files (the
+ * "Software"), to deal in the Software without restriction, including
+ * without limitation the rights to use, copy, modify, merge, publish,
+ * distribute, sublicense, and/or sell copies of the Software, and to
+ * permit persons to whom the Software is furnished to do so, subject to
+ * the following conditions:
  *
- * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
- * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
- * EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL, INDIRECT OR
- * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
- * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
- * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
- * OF THIS SOFTWARE.
+ * The above copyright notice and this permission notice (including the
+ * next paragraph) shall be included in all copies or substantial
+ * portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
+ * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
+ * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
+ * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 
 #include <math.h>
@@ -31,9 +34,11 @@
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <poll.h>
 
 #include "wayland-private.h"
 #include "test-runner.h"
+#include "test-compositor.h"
 
 static const char message[] = "Hello, world";
 
@@ -57,6 +62,7 @@ TEST(connection_create)
 
 	connection = setup(s);
 	wl_connection_destroy(connection);
+	close(s[0]);
 	close(s[1]);
 }
 
@@ -74,6 +80,7 @@ TEST(connection_write)
 	assert(memcmp(message, buffer, sizeof message) == 0);
 
 	wl_connection_destroy(connection);
+	close(s[0]);
 	close(s[1]);
 }
 
@@ -92,6 +99,7 @@ TEST(connection_data)
 	wl_connection_consume(connection, sizeof message);
 
 	wl_connection_destroy(connection);
+	close(s[0]);
 	close(s[1]);
 }
 
@@ -117,6 +125,7 @@ TEST(connection_queue)
 	assert(memcmp(message, buffer + sizeof message, sizeof message) == 0);
 
 	wl_connection_destroy(connection);
+	close(s[0]);
 	close(s[1]);
 }
 
@@ -147,8 +156,8 @@ setup_marshal_data(struct marshal_data *data)
 static void
 release_marshal_data(struct marshal_data *data)
 {
-	wl_connection_destroy(data->read_connection);
-	wl_connection_destroy(data->write_connection);
+	close(wl_connection_destroy(data->read_connection));
+	close(wl_connection_destroy(data->write_connection));
 }
 
 static void
@@ -217,7 +226,7 @@ expected_fail_marshal(int expected_error, const char *format, ...)
 {
 	struct wl_closure *closure;
 	static const uint32_t opcode = 4444;
-	static const struct wl_interface test_interface = { 
+	static const struct wl_interface test_interface = {
 		.name = "test_object"
 	};
 	static struct wl_object sender = { 0 };
@@ -235,6 +244,27 @@ expected_fail_marshal(int expected_error, const char *format, ...)
 	assert(errno == expected_error);
 }
 
+static void
+expected_fail_marshal_send(struct marshal_data *data, int expected_error,
+			   const char *format, ...)
+{
+	struct wl_closure *closure;
+	static const uint32_t opcode = 4444;
+	static struct wl_object sender = { NULL, NULL, 1234 };
+	struct wl_message message = { "test", format, NULL };
+	va_list ap;
+
+	va_start(ap, format);
+	closure = wl_closure_vmarshal(&sender, opcode, ap, &message);
+	va_end(ap);
+
+	assert(closure);
+	assert(wl_closure_send(closure, data->write_connection) < 0);
+	assert(errno == expected_error);
+
+	wl_closure_destroy(closure);
+}
+
 TEST(connection_marshal_nullables)
 {
 	struct marshal_data data;
@@ -385,13 +415,13 @@ TEST(connection_demarshal)
 	msg[0] = 400200;
 	msg[1] = 12;
 	msg[2] = 0;
-	demarshal(&data, "?s", msg, (void *) validate_demarshal_s);	
+	demarshal(&data, "?s", msg, (void *) validate_demarshal_s);
 
 	release_marshal_data(&data);
 }
 
 static void
-marshal_demarshal(struct marshal_data *data, 
+marshal_demarshal(struct marshal_data *data,
 		  void (*func)(void), int size, const char *format, ...)
 {
 	struct wl_closure *closure;
@@ -474,7 +504,7 @@ TEST(connection_marshal_alot)
 	int i;
 
 	setup_marshal_data(&data);
-	
+
 	/* We iterate enough to make sure we wrap the circular buffers
 	 * for both regular data an fds. */
 
@@ -490,6 +520,24 @@ TEST(connection_marshal_alot)
 	release_marshal_data(&data);
 }
 
+TEST(connection_marshal_too_big)
+{
+	struct marshal_data data;
+	char *big_string = malloc(5000);
+
+	assert(big_string);
+
+	memset(big_string, ' ', 4999);
+	big_string[4999] = '\0';
+
+	setup_marshal_data(&data);
+
+	expected_fail_marshal_send(&data, E2BIG, "s", big_string);
+
+	release_marshal_data(&data);
+	free(big_string);
+}
+
 static void
 marshal_helper(const char *format, void *handler, ...)
 {
@@ -528,3 +576,80 @@ TEST(invoke_closure)
 {
 	marshal_helper("suu", suu_handler, "foo", 500, 404040);
 }
+
+static void
+leak_closure(void)
+{
+	struct wl_callback *cb;
+	struct pollfd pfd;
+	struct client *c = client_connect();
+
+	cb = wl_display_sync(c->wl_display);
+	assert(cb);
+	assert(wl_display_flush(c->wl_display) > 0);
+
+	/* we don't need it, it is referenced */
+	wl_callback_destroy(cb);
+
+	pfd.fd = wl_display_get_fd(c->wl_display);
+	pfd.events = POLLIN;
+
+	test_set_timeout(2);
+	assert(poll(&pfd, 1, -1) == 1);
+
+	/* read events, but do not dispatch them */
+	assert(wl_display_prepare_read(c->wl_display) == 0);
+	assert(wl_display_read_events(c->wl_display) == 0);
+
+	/*
+	 * now we have wl_callback.done and wl_display.delete_id queued;
+	 * if we now release the queue (in wl_display_disconnect())
+	 * we should not leak memory
+	 */
+
+	client_disconnect(c);
+}
+
+TEST(closure_leaks)
+{
+	struct display *d = display_create();
+
+	client_create_noarg(d, leak_closure);
+	display_run(d);
+
+	display_destroy(d);
+}
+
+static void
+leak_after_error(void)
+{
+	struct client *c = client_connect();
+
+	/* this should return -1, because we'll send error
+	 * from server. */
+	assert(stop_display(c, 1) == -1);
+	assert(wl_display_dispatch_pending(c->wl_display) == -1);
+	assert(wl_display_get_error(c->wl_display) == ENOMEM);
+
+	/* after we got error, we have display_resume event
+	 * in the queue. It should be freed in wl_display_disconnect().
+	 * Let's see! */
+
+	wl_proxy_destroy((struct wl_proxy *) c->tc);
+	wl_display_disconnect(c->wl_display);
+	free(c);
+}
+
+TEST(closure_leaks_after_error)
+{
+	struct display *d = display_create();
+	struct client_info *cl;
+
+	cl = client_create_noarg(d, leak_after_error);
+	display_run(d);
+
+	wl_client_post_no_memory(cl->wl_client);
+	display_resume(d);
+
+	display_destroy(d);
+}
